package ast

import (
	"fmt"
	"sort"
	"strings"
	"time"
)

// String() produces a JSON-like (but not JSON) string representation of the TOML Document.
// This string version is mainly useful for testing and debugging purposes.
func (t Table) String() string {
	return NewValue(TypeTable, t).String()
}

// String() produces a JSON-like (but not JSON) string representation of the value.
// This string version is mainly useful for testing and debugging purposes.
func (parseItem Value) String() string {
	switch parseItem.Type {
	case TypeString:
		return fmt.Sprintf("%q", parseItem.Data[0])
	case TypeOffsetDateTime:
		return parseItem.Data[0].(time.Time).Format(time.RFC3339Nano)
	case TypeLocalDateTime:
		return parseItem.Data[0].(time.Time).Format("2006-01-02 15:04:05.999999999")
	case TypeLocalDate:
		return parseItem.Data[0].(time.Time).Format("2006-01-02")
	case TypeLocalTime:
		return parseItem.Data[0].(time.Time).Format("15:04:05.999999999")
	case TypeArrayOfTables:
		fallthrough
	case TypeArray:
		values := make(Key, len(parseItem.Data))
		for i, value := range parseItem.Data {
			values[i] = value.(*Value).String()
		}
		return fmt.Sprintf("[%s]", strings.Join(values, ", "))
	case TypeTable:
		pairs := parseItem.Data[0].(Table)
		keys := make([]string, len(pairs))
		i := 0
		for k := range pairs {
			keys[i] = k
			i++
		}
		sort.Strings(keys)
		values := make([]string, len(pairs))
		for i, k := range keys {
			values[i] = fmt.Sprintf("%q: %s", k, pairs[k].String())
		}
		return fmt.Sprintf("{%s}", strings.Join(values, ", "))
	default:
		return fmt.Sprintf("%v", parseItem.Data[0])
	}
}
